import { createSlice, PayloadAction } from '@reduxjs/toolkit';
import type { RootState } from './store';
import { MultiGraph } from 'graphology';
import { Attributes, SerializedGraph } from 'graphology-types';

// Define the initial state using that type
export const initialState = {
  graphologySerialized: new MultiGraph().export(),
  // schemaLayout: 'Graphology_noverlap',
};

export const querybuilderSlice = createSlice({
  name: 'querybuilder',
  // `createSlice` will infer the state type from the `initialState` argument
  initialState,
  reducers: {
    setQuerybuilderNodes: (
      state,
      action: PayloadAction<SerializedGraph<Attributes, Attributes, Attributes>>
    ) => {
      state.graphologySerialized = action.payload;
    },
    updateQBAttributeOperator: (
      state,
      action: PayloadAction<{ id: string; operator: string }>
    ) => {
      const graph = MultiGraph.from(state.graphologySerialized);
      graph.setNodeAttribute(
        action.payload.id,
        'operator',
        action.payload.operator
      );
      state.graphologySerialized = graph.export();
    },
    updateQBAttributeValue: (
      state,
      action: PayloadAction<{ id: string; value: string }>
    ) => {
      const graph = MultiGraph.from(state.graphologySerialized);
      graph.setNodeAttribute(action.payload.id, 'value', action.payload.value);
      state.graphologySerialized = graph.export();
    },
    // addQuerybuilderNode: (
    //   state,
    //   action: PayloadAction<{ id: string; attributes: Attributes }>
    // ) => {
    //   const graph = MultiGraph.from(state.graphologySerialized);
    //   graph.addNode(action.payload.id, action.payload.attributes);
    //   state.graphologySerialized = graph.export();
    // },
    // setGraphLayout: (state, action: PayloadAction<AllLayoutAlgorithms>) => {
    //   state.schemaLayout = action.payload;
    // },
  },
});

export const {
  setQuerybuilderNodes,
  updateQBAttributeOperator,
  updateQBAttributeValue,
} = querybuilderSlice.actions;

/** Select the querybuilder nodes and convert it to a graphology object */
export const selectQuerybuilderNodes = (state: RootState): MultiGraph => {
  // This is really weird but for some reason all the attributes appeared as read-only otherwise
  return MultiGraph.from(
    MultiGraph.from(state.querybuilder.graphologySerialized).export()
  );
};

// /**
//  * selects the SchemaLayout enum
//  * @param {GraphLayout} state
//  * @returns {GraphLayout} enum of type GraphLayout
//  */
// export const selectSchemaLayout = (state: RootState) =>
//   state.schema.schemaLayout;

export default querybuilderSlice.reducer;
